/*

        󑐋Z@ʃfBXvC̉p

        Copyright 2009-2011, Yoshi Hiroshi.

*/

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include "uDisp.h"
#include "uDispControl.h"
#include "logCommand.h"
#include "scope.h"
#include "switch.h"
#include "menu.h"
#include "helper.h"

volatile char selectedMenu = 0;
volatile unsigned char brightness;

char menuLabel0[] PROGMEM = "MENU";
char menuLabel1[] PROGMEM = "Trig. & Capt.";
char menuLabel2[] PROGMEM = "Capture Mode";
char menuLabel3[] PROGMEM = "Scroll Width";
char menuLabel4[] PROGMEM = "Hold Off Time";
char menuLabel5[] PROGMEM = "Brightness";
char menuLabel6[] PROGMEM = "Switch Test";
char hotLabel1[] PROGMEM = "Hold-Off Time";
char brightLabel1[] PROGMEM = "Brightness";
char checkLabel1[] PROGMEM = "    ";
char checkLabel2[] PROGMEM = "REPT";
char checkLabel3[] PROGMEM = "PUSH";
char checkLabel4[] PROGMEM = "LONG";
char checkLabel5[] PROGMEM = "REL ";
char checkLabel10[] PROGMEM = "SWITCHS:";
char checkLabel11[] PROGMEM = "EXIT:";
char checkLabel12[] PROGMEM = "LONG RED   and";
char checkLabel13[] PROGMEM = "LONG GREEN button";
char checkLabel14[] PROGMEM = "                ";
char checkLabel15[] PROGMEM = "     Please     ";
char checkLabel16[] PROGMEM = "    release     ";
char checkLabel17[] PROGMEM = "    buttons.    ";
char checkLabel21[] PROGMEM = "LEFT  ";
char checkLabel22[] PROGMEM = "RIGHT ";
char checkLabel23[] PROGMEM = "Z-IN  ";
char checkLabel24[] PROGMEM = "Z-OUT ";
char checkLabel25[] PROGMEM = "UP    ";
char checkLabel26[] PROGMEM = "DOWN  ";
char checkLabel27[] PROGMEM = "MOTION";


void showMenuPanel()
{
    uDispErase();
    uDispSetColourInt(MENU_COLOUR);
    uDispSetFontSize(UDISP_FONT_SIZE_LARGE);
    uDispDrawText_P(0, 0, menuLabel0);
    uDispSetColourInt(COLOUR_BLUE);
    uDispDrawLine(0,14, 127, 14);
    uDispSetColourInt(MENU_ITEM_COLOUR);
    uDispSetFontSize(UDISP_FONT_SIZE_LARGE);
    uDispDrawText_P(2, 2, menuLabel1);
    uDispDrawText_P(3, 2, menuLabel2);
    uDispDrawText_P(4, 2, menuLabel3);
    uDispDrawText_P(5, 2, menuLabel4);
    uDispDrawText_P(6, 2, menuLabel5);
    uDispDrawText_P(7, 2, menuLabel6);
    dispSelect(selectedMenu + 2, 0);

    utilitiesHelper();
    purgeSwitchReq();
}

char menuOperation()
{
    char result = 0;
    if(SW_IS_PUSH(SW_ZOOMIN)) {
        SW_ACK_PUSH(SW_ZOOMIN);
        if(selectedMenu < 6) {
            dispDeselect(selectedMenu + 2, 0);
            selectedMenu++;
            dispSelect(selectedMenu + 2, 0);
        }
    }
    if(SW_IS_PUSH(SW_ZOOMOUT)) {
        SW_ACK_PUSH(SW_ZOOMOUT);
        if(selectedMenu > 0) {
            dispDeselect(selectedMenu + 2, 0);
            selectedMenu--;
            dispSelect(selectedMenu + 2, 0);
        }
    }
    if(SW_IS_PUSH(SW_DOWN)) {
        SW_ACK_PUSH(SW_DOWN);
        result = selectedMenu + 1;
    }
    return result;
}

void showHoldoff(unsigned char all)
{
    static unsigned char lastValue = 0;
    if(all) {
        uDispSetColourInt(MENU_ITEM_COLOUR);
        uDispDrawRectangle(UDISP_RECT_FILL_OFF, 7, 43, 121, 77);

        uDispSetColourInt(MENU_SCOPE_BACK_COLOUR);
        uDispDrawRectangle(UDISP_RECT_FILL_ON, 9 + 112 * lcb.holdoffTime / 60, 44, 120, 76);

        uDispSetColourInt(MENU_VERTICAL_LINE_COLOUR);
        uDispDrawRectangle(UDISP_RECT_FILL_ON, 8, 44, 8 + 112 * lcb.holdoffTime / 60 , 76);
    } else {
        if(lastValue > lcb.holdoffTime) {
            uDispSetColourInt(MENU_SCOPE_BACK_COLOUR);
            uDispDrawRectangle(UDISP_RECT_FILL_ON, 
                        9 + 112 * lcb.holdoffTime / 60, 44, 
                        9 + 112 * lastValue / 60, 76);
        }
        if(lastValue < lcb.holdoffTime) {
            uDispSetColourInt(MENU_VERTICAL_LINE_COLOUR);
            uDispDrawRectangle(UDISP_RECT_FILL_ON, 
                        8 + 112 * lastValue / 60, 44, 
                        8 + 112 * lcb.holdoffTime / 60 , 76);
        }
    }

    sprintf_P(strWork, PSTR("%2dS"), lcb.holdoffTime);
    uDispSetColourInt(MENU_ITEM_COLOUR);
    uDispSetFontSize(UDISP_FONT_SIZE_LARGE);
    uDispDrawText(7, 7, strWork);
    lastValue = lcb.holdoffTime;

    holdoffHelper();
}

void showHoldoffPanel()
{
    uDispErase();
    uDispSetColourInt(MENU_COLOUR);
    uDispSetFontSize(UDISP_FONT_SIZE_LARGE);
    uDispDrawText_P(0, 0, hotLabel1);
    uDispSetColourInt(COLOUR_BLUE);
    uDispDrawLine(0,14, 127, 14);

    showHoldoff(1);
    purgeSwitchReq();
}

char holdoffOperation()
{
    char result = 0;
    if(SW_IS_PUSH(SW_LEFT) || SW_IS_REPEAT(SW_LEFT)) {
        if(lcb.holdoffTime > 1) {
            lcb.holdoffTime--;
            showHoldoff(SW_IS_PUSH(SW_LEFT));
        }
        SW_ACK_PUSH(SW_LEFT);
        SW_ACK_REPEAT(SW_LEFT);
    }
    if(SW_IS_PUSH(SW_RIGHT) || SW_IS_REPEAT(SW_RIGHT)) {
        if(lcb.holdoffTime < 60) {
            lcb.holdoffTime++;
            showHoldoff(SW_IS_PUSH(SW_RIGHT));
        }
        SW_ACK_PUSH(SW_RIGHT);
        SW_ACK_REPEAT(SW_RIGHT);
    }
    return result;
}

void showBrightness()
{
    uDispSetColourInt(MENU_SCOPE_BACK_COLOUR);
    uDispDrawRectangle(UDISP_RECT_FILL_ON, 8, 44, 120, 76);
    uDispSetColourInt(MENU_ITEM_COLOUR);
    uDispDrawRectangle(UDISP_RECT_FILL_OFF, 7, 43, 121, 77);

    uDispSetColourInt(MENU_VERTICAL_LINE_COLOUR);
    uDispDrawRectangle(UDISP_RECT_FILL_ON, 8, 44, 8 + 112 * brightness / 15 , 76);

    brightnessHelper();
}

void showBrightnessPanel()
{
    uDispErase();
    uDispSetColourInt(MENU_COLOUR);
    uDispSetFontSize(UDISP_FONT_SIZE_LARGE);
    uDispDrawText_P(0, 0, brightLabel1);
    uDispSetColourInt(COLOUR_BLUE);
    uDispDrawLine(0,14, 127, 14);

    brightness = uDispGetBrightness();
    showBrightness();
    purgeSwitchReq();
}

char brightnessOperation()
{
    char result = 0;
    if(SW_IS_PUSH(SW_LEFT)) {
        SW_ACK_PUSH(SW_LEFT);
        if(brightness > 2) {
            brightness--;
            showBrightness();
            uDispSetBrightness(brightness);
        }
    }
    if(SW_IS_PUSH(SW_RIGHT)) {
        SW_ACK_PUSH(SW_RIGHT);
        if(brightness < 15) {
            brightness++;
            showBrightness();
            uDispSetBrightness(brightness);
        }
    }
    return result;
}

void buttonCheck(unsigned char btn, unsigned char row)
{
    unsigned char sx = 72;
    static unsigned char reptFlag = 0;
    uDispSetTextBackground(1);
    if(reptFlag & _BV(row)) {
        uDispDrawText_P(row, 12, checkLabel1);
        reptFlag &= ~_BV(row);
    }
    if(SW_IS_REPEAT(btn)) {
        SW_ACK_REPEAT(btn);
        reptFlag |= _BV(row);
        uDispSetColourInt(COLOUR_CYAN);
        uDispDrawText_P(row, 12, checkLabel2);
        uDispDrawRectangle(UDISP_RECT_FILL_ON, sx, row * 12 + 1, sx+9, row * 12 + 10);
    } else
    if(SW_IS_PUSH(btn)) {
        SW_ACK_PUSH(btn);
        uDispSetColourInt(COLOUR_RED);
        uDispDrawText_P(row, 12, checkLabel3);
        uDispDrawRectangle(UDISP_RECT_FILL_ON, sx, row * 12 + 1, sx+9, row * 12 + 10);
    } else 
    if(SW_IS_LONG(btn)) {
        SW_ACK_LONG(btn);
        uDispSetColourInt(COLOUR_GREEN);
        uDispDrawText_P(row, 12, checkLabel4);
        uDispDrawRectangle(UDISP_RECT_FILL_ON, sx, row * 12 + 1, sx+9, row * 12 + 10);
    } else 
    if(SW_IS_REL(btn)) {
        SW_ACK_REL(btn);
        uDispSetColourInt(COLOUR_BLUE);
        uDispDrawText_P(row, 12, checkLabel5);
        uDispDrawRectangle(UDISP_RECT_FILL_ON, sx, row * 12 + 1, sx+9, row * 12 + 10);
    }
}

void buttonStateDisp()
{
    char val = getSwitchValue();
    uDispSetFontSize(UDISP_FONT_SIZE_LARGE);
    uDispSetColour(192, 192, 255);
    uDispDrawText_P(0, 0, checkLabel10);
    sprintf_P(strWork, PSTR("[%02X]"), val);
    uDispDrawText(0, 8, strWork);

    uDispSetFontSize(UDISP_FONT_SIZE_SMALL);
    uDispSetColour(192, 192, 255);
    uDispDrawText_P(13, 0, checkLabel11);
    uDispDrawText_P(14, 4, checkLabel12);
    uDispDrawText_P(15, 4, checkLabel13);
}

char switchTest()
{
    static char switchTestCondition = 0;
    char result = 0;
    if(SW_IN_LONG(SW_MOTION) && (SW_IN_LONG(SW_UP) || SW_IN_LONG(SW_DOWN))) {
        uDispSetFontSize(UDISP_FONT_SIZE_LARGE);
        uDispSetColour(255, 0, 0);
        uDispDrawText_P(3, 0, checkLabel14);
        uDispDrawText_P(4, 0, checkLabel15);
        uDispDrawText_P(5, 0, checkLabel16);
        uDispDrawText_P(6, 0, checkLabel17);
        uDispDrawText_P(7, 0, checkLabel14);
        while(!(SW_IN_REL(SW_MOTION) && SW_IN_REL(SW_UP) && SW_IN_REL(SW_DOWN)));
        SW_ACK_ALL(SW_MOTION);
        SW_ACK_ALL(SW_UP);
        SW_ACK_ALL(SW_DOWN);
        switchTestCondition = 0;
        result = 1;
    } else {
        if(switchTestCondition == 0) {
            switchTestCondition = 1;
            uDispErase();
            buttonStateDisp();

            uDispSetFontSize(UDISP_FONT_SIZE_LARGE);
            uDispSetColour(128, 255, 192);
            uDispDrawText_P(1, 1, checkLabel21);
            uDispDrawText_P(2, 1, checkLabel22);
            uDispDrawText_P(3, 1, checkLabel23);
            uDispDrawText_P(4, 1, checkLabel24);
            uDispDrawText_P(5, 1, checkLabel25);
            uDispDrawText_P(6, 1, checkLabel26);
            uDispDrawText_P(7, 1, checkLabel27);
        }
        uDispSetFontSize(UDISP_FONT_SIZE_LARGE);
        uDispSetColour(128, 255, 192);
        buttonCheck(SW_LEFT,    1);
        buttonCheck(SW_RIGHT,   2);
        buttonCheck(SW_ZOOMIN,  3);
        buttonCheck(SW_ZOOMOUT, 4);
        buttonCheck(SW_UP,      5);
        buttonCheck(SW_DOWN,    6);
        buttonCheck(SW_MOTION,  7);
    }
    return result;
}

