/*

        󑐋Z@ʃfBXvC̉p

        Copyright 2009-2011, Yoshi Hiroshi.

*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include "uDisp.h"
#include "switch.h"
#include "logCommand.h"

unsigned char switches[NUMSWITCHES];
unsigned int  switcheCount[NUMSWITCHES];

void initSwitchPort()
{
    DDRB &= 0xE0;
    PORTB = 0x1F;
    DDRD &= 0x3F;
    PORTD |= 0xC0;
}

void initSwitch()
{
    initSwitchPort();
    unsigned val = getSwitchValue();
    for(int inx = 0; inx < NUMSWITCHES; inx++) {
        switches[inx] = 0;
        if(val & _BV(inx)) {
            switches[inx] |= _BV(SW_ACT_PREV);
        } else {
            switches[inx] &= ~_BV(SW_ACT_PREV);
        }
    }
}

void waitForAllKeyRelease()
{
    while(getSwitchValue());
}

void enableKeyBreak(char f)
{
    if(f) {
        PCIFR |= 0x05;
        if(f == 1) {
            PCMSK0 = 0x01;
            PCMSK2 = 0x00;
        } else {
            PCMSK0 = 0x1F;
            PCMSK2 = 0xC0;
        }
        PCICR |= _BV(PCIE2) | _BV(PCIE0);
    } else {
        PCICR = 0;
    }
}

void checkIntercept()
{
    if(getSwitchValue()) {
        SET_KEY_BREAK_INT;
    }
	if(IS_LOG_BUSY)
	{
		lcb.logCounter = 1;
        lcb.breakMask = 0;
		lcb.dataReady = 1;
	}
	//LOG_STOP_INT_DISABLE;
}

ISR(PCINT0_vect)
{
    checkIntercept();
}

ISR(PCINT2_vect)
{
    checkIntercept();
}

void purgeSwitchReq()
{
    for(int inx = 0; inx < NUMSWITCHES; inx++) {
        SW_ACK_ALL(inx);
    }
}

unsigned char getSwitchValue()
{
    unsigned char val = PINB & 0x1F;
    unsigned char vald = PIND & 0xC0;
    val |= (vald >> 1) | 0x80;
    return ~val;
}

void checkSwitches()                      // process every 1mS
{
    initSwitchPort();
    unsigned val = getSwitchValue();
    for(int inx = 0; inx < NUMSWITCHES; inx++) {
        checkSwitchStatus((val & _BV(inx)), inx);
    }
}

unsigned char switchInPush(unsigned char swInx)
{
    if((switches[swInx] & (_BV(SW_ACT_PREV) | _BV(SW_ACT_CURR))) == (_BV(SW_ACT_PREV) | _BV(SW_ACT_CURR))) {
        if(switcheCount[swInx] >= SW_ON_DELAY) {
            return 1;
        }
    }
    return 0;
}

unsigned char switchInLong(unsigned char swInx)
{
    if((switches[swInx] & (_BV(SW_ACT_PREV) | _BV(SW_ACT_CURR))) == (_BV(SW_ACT_PREV) | _BV(SW_ACT_CURR))) {
        if(switcheCount[swInx] >= SW_LONG_DELAY) {
            return 1;
        }
    }
     return 0;
}

unsigned char switchInRel(unsigned char swInx)
{
    if(((switches[swInx] & (_BV(SW_ACT_PREV) | _BV(SW_ACT_CURR))) == 0)) {
        return 1;
    }
     return 0;
}

void checkSwitchStatus(unsigned char currVal, unsigned char swInx)
{
    unsigned char* sw = &switches[swInx];
    unsigned int* swCount = &switcheCount[swInx];
    if(currVal) {                           // ݒl[N̈Ɉڂ
        *sw |= _BV(SW_ACT_CURR);
    } else {
        *sw &= ~_BV(SW_ACT_CURR);
    }
    if((*sw & _BV(SW_ACT_CURR)) && (*sw & _BV(SW_ACT_PREV))) {  // no change ?
        if((*swCount)++ >= 1000) *swCount = 1000;     // ԃJE^̍XV
        if(*swCount == SW_ON_DELAY) {                   // Ԍo
            *sw |= _BV(SW_ACT_PUSH);                    //  PUSHꂽƔF
            *sw &= ~_BV(SW_ACT_RELEASE);                //  RELEASEtO|
        }

        if(*swCount == SW_LONG_DELAY) {                 // Ԍo
            *sw |= _BV(SW_ACT_LONG);                    //  LONGtOZbg
            *sw &= ~_BV(SW_ACT_RELEASE);                //  RELEASEtO|
        }

        if(*swCount == SW_REPEAT_CYCLE_DELAY) {         // Ԍo
            *sw |= _BV(SW_ACT_REPEAT);                  //  REPEATtOZbg
            *swCount = SW_REPEAT_START_DELAY;
        }

    } else {                                            // changed?
        if(*swCount >= SW_ON_DELAY) {                   // 
            if((*sw & _BV(SW_ACT_CURR)) == 0) {         //  @ꂽH
                *sw |= _BV(SW_ACT_RELEASE);             //  RELEASEtOZbg
            }
        }
        *swCount = 0;
        if(*sw & _BV(SW_ACT_CURR)) {
            *sw |= _BV(SW_ACT_PREV);
        } else {
            *sw &= ~_BV(SW_ACT_PREV);
        }
    }
}

