/*

        󑐋Z@ʃfBXvC̉p

        gZ@QOPPN@Q

        Copyright 2009-2011, Yoshi Hiroshi.

*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <stdio.h>
//#include <string.h>
#include "uDisp.h"
#include "uDispControl.h"
#include "logCommand.h"
#include "switch.h"
#include "simpleUart.h"
#include "scope.h"
#include "captureMode.h"
#include "scrollWidth.h"
#include "trigger.h"
#include "menu.h"
#include "helper.h"

/*
        motion:샂[h
 */
#define MOTION_IDLE                 0
#define MOTION_TRIGGER              11
#define MOTION_HOLDOFFTIME          12
#define MOTION_MENU                 20
#define MOTION_CAPTURE_MODE         22
#define MOTION_SCROLL_WIDTH         23
#define MOTION_BRIGHT_SET           24
#define MOTION_SWITCH_TEST          25

volatile char motion;

void dispCheck(unsigned char sx, unsigned char sy, unsigned char inten)
{
    if(inten) {
        uDispSetColourInt(COLOUR_RED);
    } else {
        uDispSetColourInt(COLOUR_DARK_RED);
    }
    uDispDrawLine(sx,     sy + 8,  sx + 1,  sy + 6);
    uDispDrawLine(sx + 1, sy + 6,  sx + 4,  sy + 11);
    uDispDrawLine(sx + 2, sy + 6,  sx + 5,  sy + 11);
    uDispDrawLine(sx + 4, sy + 11, sx + 11, sy);
    uDispDrawLine(sx + 5, sy + 11, sx + 11, sy);
    //uDispDrawLine(sx + 6, sy + 11, sx + 11, sy);
}

void dispDeselect(unsigned char row, unsigned char col)
{
    unsigned char sy = row * 12;
    unsigned char sx = col * 8;
    uDispSetColourInt(COLOUR_BLACK);
    uDispDrawRectangle(UDISP_RECT_FILL_ON, sx, sy, sx + 11, sy + 11);
}

void dispSelect(unsigned char row, unsigned char col)
{
    uDispSetColourInt(MENU_SEL_MARK_COLOUR);
    unsigned char sy = row * 12;
    unsigned char sx = col * 8 + 6;
    for(unsigned char inx = 0; inx < 6; inx++)
    {
        uDispDrawLine(sx - inx, sy + inx, sx + inx, sy + inx);
        uDispDrawLine(sx - inx, sy + 11 - inx, sx + inx, sy + 11 - inx);
    }
}

void doCaptureData()
{
    waitForAllKeyRelease();
    KEY_SCAN_TIMER_OFF;
    BUSY_LED_CAPTURE_ON;
    eraseLogBuffer();

    if(lcb.sampleSpeedUnit == 0) {
        if(lcb.sampleSpeed == 200) {
            Log_4_Cycle();
        } else if(lcb.sampleSpeed == 500) {
            Log_10_Cycle();
        }
    } else if(lcb.sampleSpeedUnit == 1) {
        if(lcb.sampleSpeed == 1) {
            Log_20_Cycle();
        } else if(lcb.sampleSpeed == 2) {
            Log_40_Cycle();
        } else if(lcb.sampleSpeed == 5) {
            Log_100_Cycle();
        } else if(lcb.sampleSpeed == 10) {
            Log_200_Cycle();
        } else {
            normalLog();
        }
    } else {
        normalLog();
    }
    BUSY_LED_CAPTURE_OFF;
    KEY_SCAN_TIMER_ON;
}


void idleState()
{
    if(SW_IS_PUSH(SW_MOTION)) {
        SW_ACK_PUSH(SW_MOTION);
        setNextCaptureTimer(0);
        motion = MOTION_TRIGGER;
        showTriggerPanel();
        if(IS_KEY_BREAK_INT) {
            CLEAR_KEY_BREAK_INT;
            waitForAllKeyRelease();
            motion = MOTION_MENU;
            showMenuPanel();
        }
    } else {
        SET_SCOPE_OPERATION_REQ;
    }
}

void triggerState()
{
    if(SW_IS_PUSH(SW_MOTION)) {
        SW_ACK_PUSH(SW_MOTION);
        motion = MOTION_MENU;
        showMenuPanel();
    } else {
        if(triggerOperation() == 1) {
            motion = MOTION_IDLE;
            SET_CAPTURE_REQ_INT;
        }
    }
}

void menuState()
{
    if(SW_IS_PUSH(SW_MOTION)) {
        SW_ACK_PUSH(SW_MOTION);
        motion = MOTION_IDLE;
        SET_CAPTURE_COMPLETE;
    } else {
        switch(menuOperation()) {
        case 0:
            break;
        case 1:
            motion = MOTION_TRIGGER;
            showTriggerPanel();
            if(IS_KEY_BREAK_INT) {
                CLEAR_KEY_BREAK_INT;
                waitForAllKeyRelease();
                motion = MOTION_MENU;
                showMenuPanel();
            }
            break;
        case 2:
            motion = MOTION_CAPTURE_MODE;
            showCaptureModePanel();
            break;
        case 3:
            motion = MOTION_SCROLL_WIDTH;
            showScrollWidthPanel();
            break;
        case 4:
            motion = MOTION_HOLDOFFTIME;
            showHoldoffPanel();
            break;
        case 5:
            motion = MOTION_BRIGHT_SET;
            showBrightnessPanel();
            break;
        case 6:
            motion = MOTION_SWITCH_TEST;
            break;
        }
    }
}

void captureState()
{
    if(SW_IS_PUSH(SW_MOTION)) {
        SW_ACK_PUSH(SW_MOTION);
        motion = MOTION_MENU;
        showMenuPanel();
    } else {
        captureModeOperation();
    }
}

void scrollWidthState()
{
    if(SW_IS_PUSH(SW_MOTION)) {
        SW_ACK_PUSH(SW_MOTION);
        motion = MOTION_MENU;
        showMenuPanel();
    } else {
        scrollWidthOperation();
    }
}

void holdoffState()
{
    if(SW_IS_PUSH(SW_MOTION)) {
        SW_ACK_PUSH(SW_MOTION);
        motion = MOTION_MENU;
        showMenuPanel();
    } else {
        holdoffOperation();
    }
}

void brightnessState()
{
    if(SW_IS_PUSH(SW_MOTION)) {
        SW_ACK_PUSH(SW_MOTION);
        motion = MOTION_MENU;
        showMenuPanel();
    } else {
        brightnessOperation();
    }
}

// every 10mS
ISR(TIMER0_COMPA_vect)
{
    checkSwitches();
    if(lcb.captureTimer > 0) {
        lcb.captureTimer--;
        if(lcb.captureTimer == 0) {
            SET_CAPTURE_REQ_INT;
        }
    }
}

void init()
{
    motion = MOTION_IDLE;

    for(unsigned int inx = 0; inx < LOG_DATA_BUFFER_SIZE; inx++) {
        logBuffer[inx] = inx;
    }

    initUart();
    initLogCommand();
    initSwitch();
    OCR0A = (10000L * 20 / 1024L) - 1; // 10mS
    TCCR0A = _BV(WGM01);
    KEY_SCAN_TIMER_ON;// TCCR0B = _BV(CS02) | _BV(CS00);     // devide by 1024
    TIMSK0 |= _BV(OCIE0A);
}

int main()
{
    unsigned char resultWork;
    if((MCUSR & _BV(EXTRF)) == 0) {
        for(char dly = 0; dly < 25; dly++) {
            _delay_ms(200);
            BUSY_LED_WAIT_TRIGGER_INV;
        }
    } else {
        //_delay_ms(500);
    }
    MCUSR = 0;

    init();
/*
    lcb.triggerMask = 0x15;
    lcb.triggerPattern = 0x04;
    Log_4_Cycle();
    Log_10_Cycle();
    Log_20_Cycle();
    Log_40_Cycle();
    Log_100_Cycle();
    Log_200_Cycle();
    normalLog();
*/
    uDispSetBrightness(0xFF);
    uDispSetBackgroundColour(0, 0, 0);
    BUSY_LED_WAIT_TRIGGER_OFF;
    BUSY_LED_CAPTURE_OFF;
    sei();

    SET_CAPTURE_COMPLETE;
    lcb.captureMode = 1;
    //lcb.triggerMask = 0x0B;
    //lcb.triggerPattern = 0x02;
    //motion = MOTION_TRIGGER;
    //showTriggerPanel();

    while(1)
    {
        if(IS_CAPTURE_REQ_INT) {
            CLEAR_CAPTURE_REQ_INT;
            if(motion == MOTION_IDLE) {
                setNextCaptureTimer(0);
                doCaptureData();
                _delay_ms(50);
                SET_CAPTURE_COMPLETE;
            }
        }

        if(IS_CAPTURE_COMPLETE) {
            CLEAR_CAPTURE_COMPLETE;
            if(motion == MOTION_IDLE) {
                setNextCaptureTimer(0);
                dispCaptureData(1);
                SET_SCOPE_OPERATION_REQ;
            }
        }

        if(IS_SCOPE_OPERATION_REQ) {
            CLEAR_SCOPE_OPERATION_REQ;
            if((resultWork = scopeOperation())) {
                if(resultWork == 1) {
                    SET_CAPTURE_REQ_INT;
                } else {
                    SET_CAPTURE_COMPLETE;
                }
            }
        }

        switch(motion) {
        case MOTION_IDLE:
            idleState();
            break;
        case MOTION_TRIGGER:
            triggerState();
            break;
        case MOTION_MENU:
            menuState();
            break;
        case MOTION_CAPTURE_MODE:
            captureState();
            break;
        case MOTION_SCROLL_WIDTH:
            scrollWidthState();
            break;
        case MOTION_HOLDOFFTIME:
            holdoffState();
            break;
        case MOTION_BRIGHT_SET:
            brightnessState();
            break;
        case MOTION_SWITCH_TEST:
            if(switchTest()) {
                motion = MOTION_MENU;
                showMenuPanel();
            }
            break;
        default:
            break;
        }
    }
}


