/*

        󑐋Z@ʃfBXvC̉p

        Copyright 2009-2011, Yoshi Hiroshi.

*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <util/delay.h>
#include <stdio.h>
#include <string.h>
#include "uDispControl.h"

unsigned char brightnessSave EEMEM;
unsigned char fSize = 2;
unsigned int  fgColour = 0xFFFF;
char strWork[12];

void uDispErase()
{
    printf_P(PSTR("\377%c%c%c"), uDispId, 1, UDISP_CMND_CLEAR);
    _delay_ms(UDISP_DELAY_LONG);
}

void uDispSetBackgroundColour(unsigned char red, unsigned char green, unsigned char blue)
{
    unsigned int cl = ((((red & 0xF8) << 5) | (green & 0xFC)) << 3) | (blue >> 3);
    printf_P(PSTR("\377%c%c%c"), uDispId, 3, UDISP_CMND_BACKGROUND);
    printf_P(PSTR("%c%c"), cl >> 8, cl & 0xFF);
    _delay_ms(UDISP_DELAY_VERY_LONG);
}

void uDispSetBackgroundColourInt(unsigned int rgb565)
{
    printf_P(PSTR("\377%c%c%c"), uDispId, 3, UDISP_CMND_BACKGROUND);
    printf_P(PSTR("%c%c"), rgb565 >> 8, rgb565 & 0xFF);
    _delay_ms(UDISP_DELAY_VERY_LONG);
}

void uDispSetTextBackground(unsigned char m)
{
    printf_P(PSTR("\377%c%c%c%c"), uDispId, 2, UDISP_CMND_TEXT_BACKGROUND, m);
    _delay_ms(UDISP_DELAY_MIDDLE);
}

void uDispSetBrightness(unsigned char b)
{
    if(b == 0xFF) {
        b = eeprom_read_byte(&brightnessSave);
    }
    if(b > 15) b = 15;
    eeprom_write_byte(&brightnessSave, b);
    printf_P(PSTR("\377%c%c%c%c%c"), uDispId, 3, UDISP_CMND_BRIGHTNESS, 2, b);
    _delay_ms(UDISP_DELAY_MIDDLE);
}

unsigned char uDispGetBrightness()
{
    return eeprom_read_byte(&brightnessSave);
}

void uDispSetFontSize(unsigned char fs)
{
    if(fs > 2) {
        fSize = 2;
    } else {
        fSize = fs;
    }
}

void uDispSetColour(unsigned char red, unsigned char green, unsigned char blue)
{
    fgColour = ((((red & 0xF8) << 5) | (green & 0xFC)) << 3) | (blue >> 3);
}

void uDispSetColourInt(unsigned int rgb565){
    fgColour = rgb565;
}

void uDispDrawText(unsigned char row, unsigned char col, char* str)
{
    unsigned char len = strlen(str) + 6;
    printf_P(PSTR("\377%c%c%c"), uDispId, len, UDISP_CMND_TEXT);
    printf_P(PSTR("%c%c%c%c%c"), col, row, fSize, fgColour >> 8, fgColour & 0xFF);
    printf(str);
    _delay_ms(UDISP_DELAY_SHORT);
}

void uDispDrawText_P(unsigned char row, unsigned char col, PGM_P str)
{
    unsigned char len = strlen_P(str) + 6;
    printf_P(PSTR("\377%c%c%c"), uDispId, len, UDISP_CMND_TEXT);
    printf_P(PSTR("%c%c%c%c%c"), col, row, fSize, fgColour >> 8, fgColour & 0xFF);
    printf_P(str);
    _delay_ms(UDISP_DELAY_SHORT);
}

void uDispDrawInteger(unsigned char row, unsigned char col, char* format, int val)
{
    sprintf(strWork, format, val); 
    unsigned char len = strlen(strWork) + 6;
    printf_P(PSTR("\377%c%c%c"), uDispId, len, UDISP_CMND_TEXT);
    printf_P(PSTR("%c%c%c%c%c"), col, row, fSize, fgColour >> 8, fgColour & 0xFF);
    printf_P(PSTR("%s"), strWork);
    _delay_ms(UDISP_DELAY_SHORT);
}

void uDispDrawLine(unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2)
{
    printf_P(PSTR("\377%c%c%c"), uDispId, 7, UDISP_CMND_LINE);
    printf_P(PSTR("%c%c%c%c%c%c"), x1, y1, x2, y2, fgColour >> 8, fgColour & 0xFF);
    _delay_ms(UDISP_DELAY_SHORT);
}

void uDispDrawRectangle(unsigned char fill, unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2)
{
    printf_P(PSTR("\377%c%c%c%c"), uDispId, 2, UDISP_CMND_FILL_RECTANGLE, fill);
    //_delay_ms(UDISP_DELAY_SHORT);
    printf_P(PSTR("\377%c%c%c"), uDispId, 7, UDISP_CMND_RECTANGLE);
    printf_P(PSTR("%c%c%c%c%c%c"), x1, y1, x2, y2, fgColour >> 8, fgColour & 0xFF);
    _delay_ms(UDISP_DELAY_MIDDLE);
}

