/*

        󑐋Z@ʃfBXvC̉p

        gZ@QOPPN@Q

        Copyright 2009-2011, Yoshi Hiroshi.

*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <stdio.h>
#include <math.h>
#include "uDispMeter.h"
#include "uDispControl.h"
#include "simpleUart.h"
#include "switch.h"

volatile char frameType = 1;
volatile int maxAdcCount = 100;
volatile char channel = 0;
volatile char hold = 0;
volatile int holdCounter;
volatile long adcSum;
volatile long adcValue;
volatile int dispValue;
volatile int adcCounter;
volatile int timerCounter;
char valStr[12] = "  1.234 V";

volatile int work;

ISR(ADC_vect)
{
    BUSY_LED_BLUE_ON;
    adcSum += ADC;
    adcCounter++;
    if(adcCounter > maxAdcCount) {
        adcCounter = 0;
        adcValue = adcSum;
        adcSum = 0;
        SET_READY_DATA;
        //BUSY_LED_RED_ON;
    }
    BUSY_LED_BLUE_OFF;
}

// every 1mS
ISR(TIMER1_COMPA_vect)
{
    ADCSRA |= _BV(ADSC);
    checkSwitches();
    if(hold) {
        if(holdCounter < 1) {
            BUSY_LED_RED_ON;
        } else {
            BUSY_LED_RED_OFF;
        }
        holdCounter++;
        if(holdCounter > 1000) holdCounter = 0;
    } else {
        BUSY_LED_RED_OFF;
        holdCounter = 0;
    }
}

void init()
{
    initUart();
    OCR1A = (1000L * 20L) - 1;    // 1mS
    TCCR1A = 0;
    TCCR1B = _BV(WGM12) | _BV(CS10);     // 1/1
    TIMSK1 |= _BV(OCIE1A);
    ADCSRA = _BV(ADEN) |_BV(ADIE) | _BV(ADPS2) | _BV(ADPS1) | _BV(ADPS0);
    ADMUX = _BV(REFS0);     // Aref = 5V, Channel = 0;
}

// =============================================================
int main()
{
    LOGSTATUS_LED_DDR_INIT;
    if((MCUSR & _BV(EXTRF)) == 0) {
        for(char dly = 0; dly < 25; dly++) {
            _delay_ms(200);
            BUSY_LED_RED_INV;
        }
    } else {
        //_delay_ms(500);
    }
    MCUSR = 0;

    init();
    initSwitch();
    uDispSetBrightness(15);
    BUSY_LED_RED_OFF;
    BUSY_LED_BLUE_OFF;
    showMeterFrame();
    sei();

    while(1)
    {
        if(IS_READY_DATA) {
            CLEAR_READY_DATA;
            if(hold) {
                showHold();
            } else {
                dispValue = (int)(adcValue * 5060L / (1024L * (long)maxAdcCount));
                showMeter(dispValue);
            }
        }
        if(SW_IS_PUSH(SW_MOTION)) {
            SW_ACK_PUSH(SW_MOTION);
            if(hold) {
                hold = 0;
            } else {
                hold = 1;
            }
        }
        if(SW_IS_PUSH(SW_UP)) {
            SW_ACK_PUSH(SW_UP);
            channel++;
            if(channel > 5) channel = 0;
            ADMUX = _BV(REFS0) | channel;
            hold = 0;
        }
        if(SW_IS_PUSH(SW_DOWN)) {
            SW_ACK_PUSH(SW_DOWN);
            if(channel) {
                channel--;
            } else {
                channel = 5;
            }
            ADMUX = _BV(REFS0) | channel;
            hold = 0;
        }
        if(SW_IS_PUSH(SW_ZOOMIN) || SW_IS_PUSH(SW_ZOOMOUT)) {
            SW_ACK_PUSH(SW_ZOOMIN);
            SW_ACK_PUSH(SW_ZOOMOUT);
            //frameType++;
            if(frameType == 2) {
                frameType = 1;
                maxAdcCount = 100;
            } else {
                frameType = 2;
                maxAdcCount = 1000;
            }
        }
    }
}

unsigned char needle[] =
{
    33,	64,	12,	46,
    34,	64,	13,	45,
    34,	63,	13,	45,
    35,	63,	14,	44,
    35,	62,	15,	43,
    36,	62,	16,	42,
    36,	61,	17,	41,
    37,	61,	18,	40,
    37,	60,	18,	39,
    38,	60,	19,	39,
    38,	59,	20,	38,
    39,	59,	21,	37,
    39,	58,	22,	36,
    40,	58,	23,	36,
    40,	58,	24,	35,
    41,	57,	25,	34,
    42,	57,	26,	34,
    42,	56,	27,	33,
    43,	56,	28,	32,
    43,	56,	29,	32,
    44,	55,	30,	31,
    45,	55,	31,	31,
    45,	55,	32,	30,
    46,	54,	33,	29,
    46,	54,	34,	29,
    47,	54,	35,	28,
    48,	53,	36,	28,
    48,	53,	37,	27,
    49,	53,	38,	27,
    50,	53,	40,	27,
    50,	52,	41,	26,
    51,	52,	42,	26,
    52,	52,	43,	25,
    52,	52,	44,	25,
    53,	52,	45,	25,
    54,	51,	46,	24,
    54,	51,	48,	24,
    55,	51,	49,	24,
    56,	51,	50,	23,
    56,	51,	51,	23,
    57,	51,	52,	23,
    58,	50,	53,	23,
    58,	50,	54,	23,
    59,	50,	56,	23,
    60,	50,	57,	22,
    60,	50,	58,	22,
    61,	50,	59,	22,
    62,	50,	60,	22,
    63,	50,	62,	22,
    63,	50,	63,	22,
    64,	50,	64,	22,
    65,	50,	65,	22,
    65,	50,	66,	22,
    66,	50,	68,	22,
    67,	50,	69,	22,
    67,	50,	70,	22,
    68,	50,	71,	22,
    69,	50,	72,	22,
    70,	50,	73,	23,
    70,	50,	75,	23,
    71,	51,	76,	23,
    72,	51,	77,	23,
    72,	51,	78,	23,
    73,	51,	79,	24,
    74,	51,	80,	24,
    74,	51,	82,	24,
    75,	52,	83,	25,
    76,	52,	84,	25,
    76,	52,	85,	25,
    77,	52,	86,	26,
    78,	52,	87,	26,
    78,	53,	88,	26,
    79,	53,	89,	27,
    80,	53,	91,	27,
    80,	53,	92,	28,
    81,	54,	93,	28,
    81,	54,	94,	29,
    82,	54,	95,	29,
    83,	55,	96,	30,
    83,	55,	97,	30,
    84,	55,	98,	31,
    85,	56,	99,	32,
    85,	56,	100,32,
    86,	56,	101,33,
    86,	57,	102,34,
    87,	57,	103,34,
    87,	58,	104,35,
    88,	58,	105,36,
    89,	58,	106,36,
    89,	59,	107,37,
    90,	59,	108,38,
    90,	60,	109,39,
    91,	60,	109,39,
    91,	61,	110,40,
    92,	61,	111,41,
    92,	62,	112,42,
    93,	62,	113,43,
    93,	63,	114,44,
    94,	63,	114,44,
    94,	64,	115,45,
    95,	64,	116,46

};

// =============================================================
void showValue(long val)
{
    static int needleIndex = -9999;
    if(val < 0) val = 0;
    if(val > 5000) val = 5000;
    sprintf(valStr, "%3d", (int)(val / 1000));
    sprintf(&valStr[3], ".%03d V", (int)(val % 1000));

    if(needleIndex >= 0) {
        uDispSetColourInt(METER_BACK_COLOUR);
        uDispDrawLine(needle[needleIndex * 4], needle[needleIndex * 4 + 1], 
                      needle[needleIndex * 4 + 2], needle[needleIndex * 4 + 3]);
    }
    uDispSetColourInt(METER_NEEDLE_COLOUR);
    needleIndex = val / 50;
    uDispDrawLine(needle[needleIndex * 4], needle[needleIndex * 4 + 1], 
                  needle[needleIndex * 4 + 2], needle[needleIndex * 4 + 3]);
    uDispSetColourInt(METER_VALUE_COLOUR);
    uDispSetFontSize(2);
    uDispDrawText( 6, 3, valStr);
}

void showChannel()
{
// Channel Number
    uDispSetColourInt(METER_CHANNEL_COLOUR);
    uDispSetFontSize(1);
    sprintf(valStr, "CH:%d", channel);
    uDispDrawText( 1, 11, valStr);
}

void showHold()
{
// Hold
    uDispSetColourInt(METER_HOLD_COLOUR);
    uDispSetFontSize(1);
    if(hold) {
        uDispDrawText( 1, 1, "HOLD");
    } else {
        uDispDrawText( 1, 1, "    ");
    }
}

void showMeter(long val)
{
    //uDispDisplayImage(0, 0, 128, 128, 0);
    showMeterPanel();
    showValue(val);
    showChannel();
    showHold();
}

unsigned char mFrame1[] =
{
    21,	54,	15,	49,
    36,	42,	32,	35,
    54,	35,	53,	27,
    74,	35,	75,	27,
    92,	41,	96,	35,
    107,54,	113,49
};

unsigned char mFrame2[] =
{
21	,	54	,	14	,	48	,
24	,	51	,	21	,	48	,
27	,	48	,	24	,	45	,
30	,	46	,	27	,	43	,
33	,	44	,	30	,	40	,
36	,	42	,	31	,	34	,
39	,	40	,	38	,	36	,
43	,	38	,	41	,	34	,
47	,	37	,	45	,	33	,
50	,	36	,	49	,	32	,
54	,	35	,	53	,	26	,
58	,	34	,	58	,	30	,
62	,	34	,	62	,	30	,
66	,	34	,	66	,	30	,
70	,	34	,	70	,	30	,
74	,	35	,	75	,	26	,
77	,	36	,	78	,	32	,
81	,	37	,	82	,	33	,
85	,	38	,	86	,	34	,
88	,	40	,	90	,	36	,
92	,	41	,	96	,	34	,
95	,	44	,	97	,	40	,
98	,	46	,	101	,	43	,
101	,	48	,	104	,	45	,
104	,	51	,	107	,	48	,
107	,	54	,	114	,	48	
};

void showMeterPanel()
{
    static char lastType = 0;
    if(lastType != frameType) {
        lastType = frameType;
        showMeterFrame();
    } else {
        unsigned char* ptr;
        char numLine;
        if(frameType == 1) {
            ptr = mFrame1;
            numLine = 6;
        } else {
            ptr = mFrame2;
            numLine = 26;
        }
        uDispSetColourInt(METER_FRAME_COLOUR);
        for(unsigned char inx = 0; inx < numLine; inx++) {
            uDispDrawLine(*ptr, *(ptr+1), *(ptr+2), *(ptr+3));
            if(inx > 0) {
                uDispDrawLine(*ptr, *(ptr+1), *(ptr-4), *(ptr-3));
            }
            ptr += 4;
        }
    }
}

void showMeterFrame()
{
    uDispSetBackgroundColourInt(METER_BACK_COLOUR);
    //uDispErase();
    uDispSetColourInt(METER_BACK_COLOUR);
    uDispDrawRectangle(0, 0, 0, 127, 127);
    uDispSetColourInt(METER_FRAME_COLOUR);
    uDispDrawRectangle(1, 0, 0, 127, 127);
    uDispSetTextBackground(1);
    uDispSetFontSize(0);
    uDispDrawText( 13, 1, "Copyright(c) 2011.");
    uDispDrawText( 14, 8, "Yoshi Hiroshi");
    showMeterPanel();
    //uDispMemoryDisplay(0, 0, 128, 128, 0);
}


